// package FlowGraph;
//import TempList;
// import Temp;
//import NodeList;
//import Node;

/**
 * A control flow graph is a directed graph in which each edge
 * indicates a possible flow of control.  Also, each node in
 * the graph defines a set of temporaries; each node uses a set of
 * temporaries; and each node is, or is not, a <strong>move</strong>
 * instruction.
 *
 * @see AssemFlowGraph
 */

public abstract class FlowGraph extends Graph {
 /**
  * The set of temporaries defined by this instruction or block 
  */
	public abstract java.util.LinkedList<Temp> def(Node node);

 /**
  * The set of temporaries used by this instruction or block 
  */
	public abstract java.util.LinkedList<Temp> use(Node node);

 /**
  * True if this node represents a <strong>move</strong> instruction,
  * i.e. one that can be deleted if def=use. 
  */
	public abstract boolean isMove(Node node);

 /**
  * Print a human-readable dump for debugging.
  */
     public void show(java.io.PrintStream out) {
	for (NodeList p=nodes(); p!=null; p=p.tail) {
	  Node n = p.head;
	  out.print(n.toString());
	  out.print(": ");
	  // for(java.util.LinkedList<Temp> q=def(n); q!=null; q=q.tail) {
	  java.util.Iterator<Temp> it = def(n).iterator();
	  while (it.hasNext()) {
	     out.print(it.next().toString());
	     out.print(" ");
	  }
	  out.print(isMove(n) ? "<= " : "<- ");
	  //for(java.util.LinkedList<Temp> q=use(n); q!=null; q=q.tail) {
	  it = use(n).iterator();
	  while (it.hasNext()) {
	     out.print(it.next().toString());
	     out.print(" ");
	  }
	  out.print("; goto ");
	  for(NodeList q=n.succ(); q!=null; q=q.tail) {
	     out.print(q.head.toString());
	     out.print(" ");
	  }
	  out.println();
	}
     }

}
    
    
